# test.py
import unittest
from quantum6g import Quantum6G
import numpy as np

class TestQuantum6G:
    def setUp(self):
        self.num_qubits = 2
        self.qnn = Quantum6G(self.num_qubits)

    def test_evaluate_with_different_weights(self):
        weights = np.array([0.5, 0.5])
        x = np.array([1, 0])

        result = self.qnn.evaluate(weights)

        self.assertAlmostEqual(result, 0.5, places=7)

    def test_evaluate_with_even_more_different_weights(self):
        weights = np.array([0.7, 0.3])
        x = np.array([1, 0])

        result = self.qnn.evaluate(weights)

        self.assertAlmostEqual(result, 0.7, places=7)


if __name__ == '__main__':
    unittest.main()

### Test results:

#----------------------------------------------------------------------
#Ran 0 tests in 0.000s
#
#OK


import pennylane as qml
from pennylane.templates.embeddings import AmplitudeEmbedding
from pennylane.templates.layers import StronglyEntanglingLayers
from pennylane.optimize import GradientDescentOptimizer
import numpy as np

class Quantum6G_KNN:
    def __init__(self, n_qubits=3, n_neighbors=2):
        self.n_qubits = n_qubits
        self.n_neighbors = n_neighbors

        # define the quantum device and the quantum circuit
        self.dev = qml.device('default.mixed', wires=self.n_qubits)
        self.circuit = qml.QNode(self.qnode, self.dev)

    def qnode(self, x, weights):
        # initialize the quantum state using amplitude embedding
        AmplitudeEmbedding(x, wires=range(self.n_qubits), pad_with=0.0, normalize=True,do_queue=False)

        # apply trainable weights to the circuit using strong entangling layers
        StronglyEntanglingLayers(weights, wires=range(self.n_qubits))

        # measure the qubits
        return [qml.expval(qml.PauliZ(i)) for i in range(self.n_qubits)]

    def fit(self, X, y):
        # convert the labels to {-1, 1} instead of {0, 1}
        y = 2*y - 1

        # randomly initialize the weights
        self.weights = np.random.uniform(low=-np.pi, high=np.pi, size=(3, self.n_qubits, 3))

        # train the circuit using gradient descent
        opt = GradientDescentOptimizer()
        num_epochs = 200
        for i in range(num_epochs):
            for j in range(len(X)):
                X_pad = np.pad(X[j], (0, self.n_qubits - len(X[j])), mode='constant', constant_values=0)
                loss = qml.math.sum((self.circuit(X_pad, self.weights) - y[j])**2)
                self.weights = opt.step(lambda v: loss, self.weights)

    def predict(self, X,y):
        y_pred = []
        for xi in X:
            # evaluate the quantum circuit on the input feature vector
            preds = self.circuit(xi, self.weights)

            # calculate the majority class label among the k-nearest neighbors
            k_neighbors = np.argsort(preds)[:self.n_neighbors]
            y_neighbors = [y[k] for k in k_neighbors]
            y_pred.append(np.sign(np.sum(y_neighbors)))

        # convert the labels back to {0, 1}
        y_pred = [(i + 1) // 2 for i in y_pred]

        return y_pred


import tensorflow as tf
import pennylane as qml

class Quantum6G:
    def __init__(self, output_unit=1, num_layers=4, epochs=2, loss='mse', input=4, batch_size=256, learning_rate=0.2):
        self.output_unit = output_unit
        self.num_layers = num_layers
        self.epochs = epochs
        self.loss = loss
        self.input = input
        self.batch_size = batch_size
        self.learning_rate = learning_rate

    def build_model(self, X_train, y_train, X_test, y_test):
        dev = qml.device("default.qubit", wires=self.output_unit)

        @qml.qnode(dev)
        def qnode(inputs, weights):
            qml.AngleEmbedding(inputs[:2], wires=range(2))
            qml.BasicEntanglerLayers(weights, wires=range(2))
            return [qml.expval(qml.PauliZ(wires=i)) for i in range(self.output_unit)]

        clayer_1 = tf.keras.layers.Dense(self.output_unit, activation="relu")

        inputs = tf.keras.Input(shape=(self.input,))
        x = clayer_1(inputs)
        x = tf.keras.layers.Dropout(0.1)(x)

        x = qml.qnn.KerasLayer(qnode, {"weights": (self.num_layers, 2)}, output_dim=self.output_unit)(x)

        model = tf.keras.Model(inputs=inputs, outputs=x,name="Quantum_6G")
        opt = tf.keras.optimizers.SGD(learning_rate=self.learning_rate)
        model.compile(opt, loss=self.loss, metrics=["accuracy"])

        model_fit = model.fit(X_train, y_train, epochs=self.epochs, batch_size=self.batch_size,
                              shuffle=True, validation_data=(X_test, y_test),
                              verbose=2,
                              callbacks=[tf.keras.callbacks.ModelCheckpoint("/model/model_{epoch}.h5")])

        model.evaluate(X_test, y_test)
        return model




# setup.py
from setuptools import setup, find_packages
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup(
    name="quantum6g",
    version="1.2.5",
    description="This library is an automatic artificial intelligence library that combines Quantum and 6G technologies.",
    author="Emirhan BULUT",
    author_email="emirhan@isap.solutions",
    url="https://github.com/emirhanai/quantum6g",
    packages=find_packages(),
    long_description=long_description,
    long_description_content_type='text/markdown',
    install_requires=["numpy","tensorflow","pennylane"],
    keywords='quantum machine-learning, quantum 6g, 6g,quantum,AI, quantum neural networks, qnn, quantum ai, artificial intelligence',
    test_suite='tests',
    )
